@@ -73,7 +73,7 @@ void usage(const Protocol& prot) {
   STD_cout << "  Transformation (magnitude, logarithm, negation, inversion, or acos) of one data set:" << STD_endl;
   STD_cout << "       micalc -if <input-file> -op <operation(abs,log,-,/,acos)> -of <output-file>" << STD_endl;
   STD_cout << "  Statistics of one data set:" << STD_endl;
-  STD_cout << "       micalc -if <input-file> [-mean <time-mean-file>] [-stdev <time-stdev-file>] [-fluct <relative-time-stdev-file>] [-tcourse <time-course of all voxels>] [-hist <histogram> -histmin <minval> -histmax <maxval> -histslots <numofslots> -rightstairs -histfract]" << STD_endl;
+  STD_cout << "       micalc -if <input-file> [-mean <time-mean-file>] [-stdev <time-stdev-file>] [-fluct <relative-time-stdev-file>] [-tcourse <average time course of all voxels>] [-hist <histogram> -histmin <minval> -histmax <maxval> -histslots <numofslots> -rightstairs -histfract]" << STD_endl;
   STD_cout << "Extra options:" << STD_endl;
   STD_cout << "\t-mask <Mask for data>" << STD_endl;
   STD_cout << "File read options:" << STD_endl;
