@@ -50,16 +50,26 @@ class MiViewFmri : public JcampDxBlock {
   JDXstring fmrifile;
   JDXstring maskfile;
   JDXbool bonferr;
+  JDXbool hrf;
   JDXfloat corr;
+  JDXfloat zscore;
+  JDXint designavg;
   JDXint neighb;
   JDXstring sigchange;
+  JDXfloat zsum;
+  JDXint zcount;
+  JDXstring zaverage;
   JDXfloatArr design;
   JDXfloatArr tcourse;
+  JDXfloatArr sigcourse;
+  JDXstring dumptcourse;
+  JDXstring dumpzmap;
 
   FmriData* data;
   mutable farray overlay_map;
   mutable farray toberemoved;
   bool valid;
+
 };
 
 
