@@ -209,6 +209,8 @@ MiViewView::MiViewView(QWidget *parent) : QWidget(parent) {
   guiprops_cache.pixmap.autoscale=false;
   guiprops_cache.pixmap.color=mopts.color;
   guiprops_cache.pixmap.overlay_rectsize=mopts.maprect;
+  guiprops_cache.pixmap.overlay_minval=mopts.maplow;
+  guiprops_cache.pixmap.overlay_maxval=mopts.mapupp;
 
   has_olm=false;
   if(mopts.mapfile!="") {
@@ -217,8 +219,6 @@ MiViewView::MiViewView(QWidget *parent) : QWidget(parent) {
       Protocol protdummy;
       if(!filterchain.apply(protdummy, mapdata)) exit(-1);
       guiprops_cache.pixmap.overlay_map=mapdata;
-      guiprops_cache.pixmap.overlay_minval=mopts.maplow;
-      guiprops_cache.pixmap.overlay_maxval=mopts.mapupp;
       has_olm=true;
 //      gp.pixmap.overlay_map.normalize();
     }
@@ -401,7 +401,7 @@ void MiViewView::showTimecourse() {
 
 void MiViewView::showProtocol() {
   file->prot.set_parmode(noedit);
-  new JDXwidgetDialog(file->prot,2,this,false);
+  new JDXwidgetDialog(file->prot,2,this,false,true);
 }
 
 
