@@ -834,7 +834,9 @@ void Data<T,N_rank>::congrid(const TinyVector<int,N_rank>& newshape, const TinyV
   Log<OdinData> odinlog("Data","congrid");
   for(int i=0;i<N_rank;i++) {
     ODINLOG(odinlog,normalDebug) << "oldshape/newshape(" << i << ")=" << Array<T,N_rank>::shape()(i) << "/" << newshape(i) << STD_endl;
-    if(subpixel_shift) ODINLOG(odinlog,normalDebug) << "subpixel_shift(" << i << ")=" << (*subpixel_shift)(i) << STD_endl;
+    if(subpixel_shift) {
+      ODINLOG(odinlog,normalDebug) << "subpixel_shift(" << i << ")=" << (*subpixel_shift)(i) << STD_endl;
+    }
   }
 
   for(int irank=0; irank<N_rank; irank++) {
