@@ -46,7 +46,7 @@ int FileFormat::write(const Data<float,4>& data, const STD_string& filename, con
 int FileFormat::write(const FileIO::ProtocolDataMap& pdmap, const STD_string& filename, const FileWriteOpts& opts) {
   int result=0;
 
-  svector fnames=create_unique_filenames(filename, pdmap);
+  svector fnames=create_unique_filenames(filename, pdmap, opts.fnamepar);
 
   int ifile=0;
   for(FileIO::ProtocolDataMap::const_iterator pdit=pdmap.begin(); pdit!=pdmap.end(); ++pdit) {
@@ -103,7 +103,7 @@ void FileFormat::format_error(const STD_string& filename) {
 }
 
 
-svector FileFormat::create_unique_filenames(const STD_string& filename, const FileIO::ProtocolDataMap& pdmap) {
+svector FileFormat::create_unique_filenames(const STD_string& filename, const FileIO::ProtocolDataMap& pdmap, const STD_string& par) {
   Log<FileIO> odinlog("FileFormat","create_unique_filenames");
 
   unsigned int nnames=pdmap.size();
@@ -124,17 +124,29 @@ svector FileFormat::create_unique_filenames(const STD_string& filename, const Fi
     maxseries=STD_max(maxseries,serNum);
   }
 
+  svector pars;
+  pars=tokens(par);
+  int npars=pars.size();
+
   // Create vector of strings "S<serNumber>[_<serDescription>]"
   STD_map<STD_string,unsigned int> namescount;
   svector unique_names; unique_names.resize(nnames);
   unsigned int iname=0;
   for(FileIO::ProtocolDataMap::const_iterator pdit=pdmap.begin(); pdit!=pdmap.end(); ++pdit) {
     pdit->first.study.get_Series(serDesc,serNum);
-    for(unsigned int i=0;i<serDesc.length();i++)//replace every space in the description by "_" (gives better filenames)
-      if(serDesc[i]==' ' || serDesc[i]==':' || serDesc[i]=='/' || serDesc[i]=='&')serDesc[i]='_';
-
+    for(unsigned int i=0;i<serDesc.length();i++) { //replace odd characters in the description by "_" (gives better filenames)
+      const char cr=serDesc[i];
+      if(cr==' ' || cr==':' || cr==';' || cr=='/' || cr=='&') serDesc[i]='_';
+    }
     STD_string str("S"+itos(serNum,maxseries));
     if(serDesc!="") str+=STD_string("_")+serDesc;
+    
+    if(npars) {
+      for(int ipar=0; ipar<npars; ipar++) {
+        str+=STD_string("_")+rmblock(pdit->first.printval(pars[ipar]), "\n", ""); // truncate after first newline
+      }
+    }
+    
     if(namescount.find(str)==namescount.end()) namescount[str]=0;
     else namescount[str]++;
     unique_names[iname]=str;
@@ -205,13 +217,6 @@ float FileFormat::voxel_extent(const Geometry& geometry, direction direction, in
   return result;
 }
 
-dvector FileFormat::data_diagonal(const Geometry& geometry,int voxels_read,int voxels_phase) {
-  dvector voxeldist(3);
-  voxeldist[0]=geometry.get_FOV(readDirection)-voxel_extent(geometry,readDirection,voxels_read);
-  voxeldist[1]=geometry.get_FOV(phaseDirection)-voxel_extent(geometry,phaseDirection,voxels_phase);
-  voxeldist[2]=geometry.get_FOV(sliceDirection)-geometry.get_sliceThickness();
-  return geometry.get_gradrotmatrix()*voxeldist;
-}
 
 STD_map<STD_string,FileFormat*> FileFormat::formats;
 
@@ -395,8 +400,9 @@ struct ImageKey : public UniqueIndex<ImageKey> {
   double acquisition_time;
   double slice_loc;
   STD_string filename;
+  STD_string datatype;
 
-  ImageKey(double tt, double sl, const STD_string fn) : acquisition_time(tt), slice_loc(sl), filename(fn)  {}
+  ImageKey(double tt, double sl, const STD_string& fn, const STD_string& dt) : acquisition_time(tt), slice_loc(sl), filename(fn), datatype(dt)  {}
 
   bool operator < (const ImageKey& ik) const {
     if(slice_loc!=ik.slice_loc) return (slice_loc<ik.slice_loc); // slice location has highest priority
@@ -437,6 +443,7 @@ int FileIO::read_dir(ProtocolDataMap& pdmap, const STD_string& dirname, const Fi
   opts_copy.dset="";
   if(progmeter)progmeter->new_task( nfiles,"reading files from directory");
 
+  
   for(unsigned int i=0; i<nfiles; i++) {
 
     ProtocolDataMap onepdmap;
@@ -464,6 +471,7 @@ int FileIO::read_dir(ProtocolDataMap& pdmap, const STD_string& dirname, const Fi
 
         KeyImageMap& imgmap=oneimagemap[onefileprot]; // Get/insert key-image map for this protocol
 
+
         // Check x- and y-size
         KeyImageMap::const_iterator it=imgmap.begin();
         if(it!=imgmap.end()) { // only check size if there already is an entry in the key-image map
@@ -477,6 +485,9 @@ int FileIO::read_dir(ProtocolDataMap& pdmap, const STD_string& dirname, const Fi
           }
         }
 
+
+        STD_string dtype_prot=onefileprot.system.get_data_type();
+
         // Insert repetitions and slices separately into map
         for(int irep=0; irep<onefiledata.extent(timeDim); irep++) {
           for(int islice=0; islice<onefiledata.extent(sliceDim); islice++) {
@@ -484,7 +495,7 @@ int FileIO::read_dir(ProtocolDataMap& pdmap, const STD_string& dirname, const Fi
             double acqstart=onefileprot.seqpars.get_AcquisitionStart()+double(irep)*onefileprot.seqpars.get_RepetitionTime();
             double sliceloc=onefileprot.geometry.get_sliceOffsetVector()[islice];
             ODINLOG(odinlog,normalDebug) << "acqstart/sliceloc=" << acqstart << "/" << sliceloc << STD_endl;
-            Data<float,2>& dataref=imgmap[ImageKey(acqstart,sliceloc,dirfilename)];
+            Data<float,2>& dataref=imgmap[ImageKey(acqstart,sliceloc,dirfilename,dtype_prot)];
             dataref.resize(onefiledata.extent(phaseDim),onefiledata.extent(readDim));
             dataref=onefiledata(irep,islice,all,all);
 
@@ -499,7 +510,8 @@ int FileIO::read_dir(ProtocolDataMap& pdmap, const STD_string& dirname, const Fi
 
   ODINLOG(odinlog,normalDebug) << "mem(read): " << Profiler::get_memory_usage() << STD_endl;
 
-
+  
+  
   // Iterate over separate protocols to combine data
   int protindex=0;
   for(STD_map<Protocol, KeyImageMap>::iterator it=oneimagemap.begin(); it!=oneimagemap.end(); ++it) {
@@ -521,14 +533,33 @@ int FileIO::read_dir(ProtocolDataMap& pdmap, const STD_string& dirname, const Fi
 
     // Get slice offsets and min acq time by iterating over images
     STD_list<float> slice_offsets;
+    STD_list<STD_string> datatypes;
     double min_acqstart=imgit->first.acquisition_time;
     double max_acqstart=imgit->first.acquisition_time;
     for(imgit=imgmap.begin(); imgit!=imgmap.end(); ++imgit) {
       slice_offsets.push_back(imgit->first.slice_loc);
+      datatypes.push_back(imgit->first.datatype);
       min_acqstart=STD_min(min_acqstart,imgit->first.acquisition_time);
       max_acqstart=STD_max(max_acqstart,imgit->first.acquisition_time);
     }
 
+
+    // Find datatype with highest resolution in final dataset and use it for all datasets
+    datatypes.sort();
+    datatypes.unique();
+    STD_string dtype=TypeTraits::type2label((float)0); // default  
+    if(find(datatypes.begin(),datatypes.end(),TypeTraits::type2label((u8bit) 0))!=datatypes.end()) dtype=TypeTraits::type2label((u8bit)0);
+    if(find(datatypes.begin(),datatypes.end(),TypeTraits::type2label((s8bit) 0))!=datatypes.end()) dtype=TypeTraits::type2label((s8bit)0);
+    if(find(datatypes.begin(),datatypes.end(),TypeTraits::type2label((u16bit)0))!=datatypes.end()) dtype=TypeTraits::type2label((u16bit)0);
+    if(find(datatypes.begin(),datatypes.end(),TypeTraits::type2label((s16bit)0))!=datatypes.end()) dtype=TypeTraits::type2label((s16bit)0);
+    if(find(datatypes.begin(),datatypes.end(),TypeTraits::type2label((u32bit)0))!=datatypes.end()) dtype=TypeTraits::type2label((u32bit)0);
+    if(find(datatypes.begin(),datatypes.end(),TypeTraits::type2label((s32bit)0))!=datatypes.end()) dtype=TypeTraits::type2label((s32bit)0);
+    if(datatypes.size()>1) {
+      ODINLOG(odinlog,loglevel()) << "Using single datatype " << dtype << " for images with datatypes=" << svector(list2vector(datatypes)).printbody() << STD_endl;
+    }
+    prot.system.set_data_type(dtype);
+
+
     slice_offsets.sort();
     slice_offsets.unique();
     shape(sliceDim)=slice_offsets.size();
@@ -609,7 +640,7 @@ int FileIO::autowrite(const ProtocolDataMap& pdmap, const STD_string& filename,
 
 
   if(opts.wprot!="") {
-    svector protfnames=FileFormat::create_unique_filenames(opts.wprot, pdmap);
+    svector protfnames=FileFormat::create_unique_filenames(opts.wprot, pdmap, opts.fnamepar);
     unsigned int ifile=0;
     for(ProtocolDataMap::const_iterator pdit=pdmap.begin(); pdit!=pdmap.end(); ++pdit) {
       ODINLOG(odinlog,loglevel()) << "Storing protocol in file " << protfnames[ifile] << STD_endl;
@@ -627,7 +658,7 @@ int FileIO::autowrite(const ProtocolDataMap& pdmap, const STD_string& filename,
   int result=0;
   if(opts.split) {
 
-    svector splitfnames=FileFormat::create_unique_filenames(filename, pdmap);
+    svector splitfnames=FileFormat::create_unique_filenames(filename, pdmap, opts.fnamepar);
 
     unsigned int ifile=0;
     for(ProtocolDataMap::const_iterator pdit=pdmap.begin(); pdit!=pdmap.end(); ++pdit) {
