@@ -22,6 +22,7 @@
 
 enum vista_dialect{unknown,common,odin,lipsia,timecourse,dwi};
 
+
 /////////////////////////////////////////////////////////////
 
 class LipsiaImages: public STD_list<STD_pair<Protocol,VImage> >{
@@ -71,6 +72,8 @@ struct VistaFormat : public FileFormat {
 
   STD_string description() const {return "Vista";}
 
+
+
   svector suffix() const  {
     svector result; result.resize(1);
     result[0]="v";
@@ -97,6 +100,14 @@ struct VistaFormat : public FileFormat {
     pdmap.insert(pdpair);
   }
 
+  static dvector data_diagonal(const Geometry& geometry,int voxels_read,int voxels_phase) {
+    dvector voxeldist(3);
+    voxeldist[0]=geometry.get_FOV(readDirection)-voxel_extent(geometry,readDirection,voxels_read);
+    voxeldist[1]=geometry.get_FOV(phaseDirection)-voxel_extent(geometry,phaseDirection,voxels_phase);
+    voxeldist[2]=geometry.get_FOV(sliceDirection)-geometry.get_sliceThickness();
+    return geometry.get_gradrotmatrix()*voxeldist;
+  }
+
   template<typename T,VRepnKind K,typename R> static bool readAttr(const VAttrList &attr, const char *name,R &ret){
     T dummy;
     if(VGetAttr(attr, name, NULL, K, &dummy)==VAttrFound){ret=dummy;return true;}
