@@ -13,11 +13,17 @@ bool str2range(const STD_string& str, Range& range, int srcsize) {
   if(toks.size()==2) {range=Range(atoi(toks[0].c_str()),atoi(toks[1].c_str()),increment); result=true;}
   if(toks.size()==1) {range=Range(atoi(toks[0].c_str()),atoi(toks[0].c_str()),increment); result=true;}
 
+  if(!result) {
+    ODINLOG(odinlog,errorLog) << "Error parsing range string >" << str << "<" << STD_endl;
+  }
+
   if(result) {
     if(range.first()>range.last()) result=false;
     if(range.first()<0 || range.first()>=srcsize) result=false;
     if(range.last()<0  || range.last()>=srcsize)  result=false;
-    if(!result) ODINLOG(odinlog,errorLog) << "Values " << range << " out of allowable range (0," << (srcsize-1) << ")" << STD_endl;
+    if(!result) {
+      ODINLOG(odinlog,errorLog) << "selected " << range << " out of allowable range (0," << (srcsize-1) << ")" << STD_endl;
+    }
   }
   return result;
 }
