@@ -7,12 +7,12 @@
 #include "filter_lowpass.h"
 #include "filter_mask.h"
 #include "filter_merge.h"
-#include "filter_mip.h"
 #include "filter_nan.h"
 #include "filter_resize.h"
 #include "filter_rot.h"
 #include "filter_flip.h"
 #include "filter_range.h"
+#include "filter_reduction.h"
 #include "filter_reslice.h"
 #include "filter_scale.h"
 #include "filter_shift.h"
@@ -44,6 +44,7 @@ void FilterStep::create_templates(STD_list<FilterStep*>& result) {
   result.push_back(new FilterLowPass());
 
   result.push_back(new FilterGenMask());
+  result.push_back(new FilterAutoMask());
   result.push_back(new FilterUseMask());
 
   result.push_back(new FilterNaN());
@@ -70,7 +71,9 @@ void FilterStep::create_templates(STD_list<FilterStep*>& result) {
   result.push_back(new FilterMerge());
   result.push_back(new FilterSplice());
 
-  result.push_back(new FilterMip());
+  result.push_back(new FilterReduction<minip>());
+  result.push_back(new FilterReduction<maxip>());
+  result.push_back(new FilterReduction<proj>());
 
 }
 
