@@ -5,6 +5,17 @@
 #include <locale.h>
 #endif
 
+void set_c_locale() {
+  Log<JcampDx> odinlog("JcampDxBlock","set_c_locale");
+
+#ifdef HAVE_LOCALE_H
+ // set locale to default before reading/writing blocks
+ // to get decimal point on all platforms
+  const char* retval=setlocale(LC_NUMERIC, "C");
+  ODINLOG(odinlog,normalDebug) << "LC_NUMERIC=" << retval << STD_endl;
+#endif
+}
+
 JcampDxBlock::JcampDxBlock(const STD_string& title,compatMode mode) : garbage(0), embed(true) {
   Log<JcampDx> odinlog(title.c_str(),"JcampDxBlock(title)");
   set_label(title);
@@ -234,6 +245,7 @@ STD_string JcampDxBlock::get_parx_code(parxCodeType type, const ParxEquiv& equiv
 
 int JcampDxBlock::load(const STD_string &filename) {
   Log<JcampDx> odinlog(this,"load");
+  set_c_locale(); // Make sure we have right locale
   STD_string blockstr;
   int retval=::load(blockstr,filename);
   ODINLOG(odinlog,normalDebug) <<  "retval(" << filename << ")=" << retval << STD_endl;
@@ -263,6 +275,8 @@ int JcampDxBlock::parseblock(const STD_string& source) {
 int JcampDxBlock::write(const STD_string &filename) const {
   Log<JcampDx> odinlog(this,"write");
 
+  set_c_locale(); // Make sure we have right locale
+
   STD_ofstream outfile(filename.c_str());
   JcampDxBlock::print2stream(outfile);
   outfile.close();
@@ -577,13 +591,7 @@ STD_string JcampDxBlock::get_cmdline_usage(const STD_string& lineprefix) const {
 
 void JcampDxBlock::init_static() {
   Log<JcampDx> odinlog("JcampDxBlock","init_static");
-
-#ifdef HAVE_LOCALE_H
- // set locale to default before reading/writing blocks
- // to get decimal point on all platforms
-  const char* retval=setlocale(LC_NUMERIC, "C");
-  ODINLOG(odinlog,normalDebug) << "locale set to " << retval << STD_endl;
-#endif
+  set_c_locale();
 }
 
 
