@@ -422,11 +422,6 @@ bool RecoController::autorecipe(const RecoCoord& mask, STD_string& result) const
   if(!get_kspace_sampling_pattern<line3d,line>(mask,reduction_factor3d, partial_fourier3d)) return false;
   ODINLOG(odinlog,normalDebug) << "reduction_factor3d/partial_fourier3d=" << reduction_factor3d << "/" << partial_fourier3d << STD_endl;
 
-  if(partial_fourier && partial_fourier3d) {
-    ODINLOG(odinlog,warningLog) << "Not yet implemented: partial_fourier && partial_fourier3d" << STD_endl;
-    return false;
-  }
-
 
   unsigned int max_reduction_factor=STD_max(reduction_factor,reduction_factor3d);
   unsigned int reduction_factor_prot=protocol().seqpars.get_ReductionFactor();
@@ -544,9 +539,14 @@ bool RecoController::autorecipe(const RecoCoord& mask, STD_string& result) const
 
   STD_string fftstr="fft | ";
   if(!has_traj && !disableHomodyne) {
-    if(has_relcenter_cache) fftstr="homodyneread | ";
-    if(partial_fourier) fftstr="homodyne | ";
-    if(partial_fourier3d) fftstr="homodyne3d | ";
+    unsigned int homodyne_dims=0;
+    if(has_relcenter_cache)            {fftstr="homodyneread | "; homodyne_dims++;}
+    if (partial_fourier && !has_blade) {fftstr="homodyne | ";     homodyne_dims++;}
+    if(partial_fourier3d)              {fftstr="homodyne3d | ";   homodyne_dims++;}
+    if(homodyne_dims>1) { 
+      ODINLOG(odinlog,errorLog) << "Not yet implemented: partial Fourier reconstruction in more than one direction" << STD_endl;
+      return false;
+    }
   }
 
   STD_string offsetstr="offset | ";
@@ -585,12 +585,20 @@ bool RecoController::autorecipe(const RecoCoord& mask, STD_string& result) const
     STD_string keyholeopt;
     if(keyhole) keyholeopt="(true)";
 
+    // partial fourier reconstruction has to be applied before combining the blades!
+    STD_string bladefftstr = "";
+    if (mfreqend!="") bladefftstr = "fft | ";
+    if (!disableHomodyne && partial_fourier) {
+      if (mfreqend!="") bladefftstr = "homodyne | ";
+      else bladefftstr = "homodyne | fft(false) | ";
+    }
+
     STD_string bladecorrstr="bladecorr | ";
 
-    if(mfreqend!="") bladecorrstr="fft | "+mfreqend+"magn | "+adcdeapodize_blade+"fft(false) | ";
+    if(mfreqend!="") bladecorrstr=mfreqend+"magn | "+adcdeapodize_blade+"fft(false) | ";
     mfreqend=""; // do not use in rest of pipeline
 
-    bladestr=bladecorrstr+swstr+"cyclecoll | bladecomb"+keyholeopt+" | grid2d | line3dcoll | ";
+    bladestr=bladefftstr+bladecorrstr+swstr+"cyclecoll | bladecomb"+keyholeopt+" | grid2d | line3dcoll | ";
     if(!dumpKspace) deapodizestr="deapodize2d | ";
     gridstr="gridcut | ";
   }
