@@ -100,7 +100,7 @@ class RecoController {
 /**
   * Returns the scan protocol.
   */
-  Protocol protocol() const;
+  const Protocol& protocol() const {return prot_cache;}
 
 
 /**
@@ -165,12 +165,15 @@ class RecoController {
   JDXstring recipeFile;
   JDXstring select;
   JDXint    timeout;
+  JDXint    maxrecursion;
   JDXbool   conjPhaseFT;
   JDXbool   slidingWindow;
+  JDXbool   keyhole;
   JDXbool   sepChannels;
   JDXbool   dumpKspace;
   JDXbool   disableHomodyne;
   JDXbool   disableMultiFreq;
+  JDXbool   disableSliceTime;
   JDXbool   phaseCourse;
   JDXstring preproc3d;
   JDXstring postproc3d;
@@ -222,9 +225,10 @@ class RecoController {
   mutable bool has_templcorr_cache[n_templateTypes];
   mutable bool has_navigator_cache[n_navigatorTypes];
   mutable bool has_traj;
-  mutable float oversampling_cache;
+  mutable bool has_oversampling_cache;
+  mutable bool has_relcenter_cache;
 
-  Protocol dummyprot;
+  Protocol prot_cache;
 
   volatile bool status; // Although it is used in different threads, we will assume that reads/writes to bool are atomic
 
