@@ -301,6 +301,26 @@ struct RecoCoord {
   */
   float overSamplingFactor;
 
+/**
+  * Relative position of k-space center in readout direction
+  */
+  float relCenterPos;
+
+/**
+  * First index in readout direction actually measured
+  */
+  unsigned short adcstart;
+
+/**
+  * Last index in readout direction actually measured
+  */
+  unsigned short adcend;
+
+/**
+  * Scale factor for gridding reconstruction
+  */
+  float gridScaleFactor;
+
 /**
   * Echo-train index in a multi-echo readout, e.g. an CPMG or EPI echo train.
   * Use this for non-phasecorr template scans and set echo index to zero.
@@ -321,6 +341,10 @@ struct RecoCoord {
     weightVec=0;
     dwellTime=0.0;
     overSamplingFactor=1.0;
+    relCenterPos=0.5;
+    adcstart=0;
+    adcend=0;
+    gridScaleFactor=1.0;
     echopos=0;
     echotrain=0;
     index[readout].set_mode(RecoIndex::collected); // readout is already collected for ADCs
