@@ -128,9 +128,11 @@ void RecoStep::create_templates(STD_list<RecoStep*>& result) {
   result.push_back(new RecoFMRI());
   result.push_back(new RecoFreqComb());
   result.push_back(new RecoGrappaWeights<line,line3d,1,0>());
-  result.push_back(new RecoGrappaWeights<line,line3d,1,0, RecoDim<3, repetition, freq, userdef> >()); // Assume only one GRAPPA template for all userdef indices
+//  result.push_back(new RecoGrappaWeights<line,line3d,1,0, RecoDim<3, repetition, freq, userdef> >()); // Assume only one GRAPPA template for all userdef indices
+  result.push_back(new RecoGrappaWeights<line,line3d,1,0, RecoDim<2, repetition, freq> >());
   result.push_back(new RecoGrappaWeights<line3d,line,0,1>());
-  result.push_back(new RecoGrappaWeights<line3d,line,0,1, RecoDim<3, repetition, freq, userdef> >()); // Assume only one GRAPPA template for all userdef indices
+//  result.push_back(new RecoGrappaWeights<line3d,line,0,1, RecoDim<3, repetition, freq, userdef> >()); // Assume only one GRAPPA template for all userdef indices
+  result.push_back(new RecoGrappaWeights<line3d,line,0,1, RecoDim<2, repetition, freq> >());
   result.push_back(new RecoGrappa<line,line3d,1,0>());
   result.push_back(new RecoGrappa<line3d,line,0,1>());
   result.push_back(new RecoGrid<1>());
