@@ -18,6 +18,8 @@ void SeqSimMagsi::common_init() {
 
   online=true;
 
+  time_intervals_cache=0;
+
   xpos_cache=0;
   ypos_cache=0;
   zpos_cache=0;
@@ -59,6 +61,7 @@ void SeqSimMagsi::common_init() {
 
 
 void SeqSimMagsi::outdate_simcache() {
+  if(time_intervals_cache) delete[] time_intervals_cache; time_intervals_cache=0;
   if(xpos_cache) delete[] xpos_cache; xpos_cache=0;
   if(ypos_cache) delete[] ypos_cache; ypos_cache=0;
   if(zpos_cache) delete[] zpos_cache; zpos_cache=0;
@@ -166,13 +169,25 @@ SeqSimMagsi& SeqSimMagsi::resize(unsigned int xsize, unsigned int ysize,  unsign
 
 
 SeqSimMagsi& SeqSimMagsi::reset_magnetization() {
-  for(unsigned int i=0; i<Mx.length(); i++) {
+  unsigned int i;
+  for(i=0; i<Mx.length(); i++) {
     Mx[i]=initial_vector[0];
     My[i]=initial_vector[1];
     Mz[i]=initial_vector[2];
     Mamp[i]=0.0;
     Mpha[i]=0.0;
   }
+
+  for(unsigned int k=0; k<4; k++) {
+    if(dMx[k]) {
+      for(i=0; i<oneframe_size_cache; i++) {
+        dMx[k][i]=0.0;
+        dMy[k][i]=0.0;
+        dMz[k][i]=0.0;
+      }
+    }
+  }
+
   return *this;
 }
 
@@ -200,6 +215,7 @@ SeqSimMagsi& SeqSimMagsi::set_spat_rotmatrix(const RotMatrix& rotmatrix) {
 
 
 SeqSimMagsi& SeqSimMagsi::MampMpha2MxMy() {
+/*
   unsigned int nx=Mx.size(xDim);
   unsigned int ny=Mx.size(yDim);
   unsigned int nz=Mx.size(zDim);
@@ -207,6 +223,9 @@ SeqSimMagsi& SeqSimMagsi::MampMpha2MxMy() {
 
   Mx.redim (nfreq,nz,ny,nx);
   My.redim (nfreq,nz,ny,nx);
+*/
+
+  My.redim(Mx.get_extent());
 
   for(unsigned int i=0; i<get_total_size(); i++) {
     Mx[i]=float(Mamp[i])*cos(PII/180.0*float(Mpha[i]));
@@ -220,6 +239,7 @@ SeqSimMagsi& SeqSimMagsi::MampMpha2MxMy() {
 
 
 SeqSimMagsi& SeqSimMagsi::MxMy2MampMpha() {
+/*
   unsigned int nx=Mx.size(xDim);
   unsigned int ny=Mx.size(yDim);
   unsigned int nz=Mx.size(zDim);
@@ -227,6 +247,10 @@ SeqSimMagsi& SeqSimMagsi::MxMy2MampMpha() {
 
   Mamp.redim (nfreq,nz,ny,nx);
   Mpha.redim (nfreq,nz,ny,nx);
+*/
+
+  Mamp.redim(Mx.get_extent());
+  Mpha.redim(Mx.get_extent());
 
   for(unsigned int i=0; i<get_total_size(); i++) {
     Mamp[i]=norm(Mx[i],My[i]);
@@ -290,54 +314,89 @@ void SeqSimMagsi::prepare_simulation(const Sample& sample, CoilSensitivity* tran
 
   outdate_simcache(); // delete old pointers
 
+
   ndim extent=sample.get_extent();
-  unsigned int n=extent.total();
   unsigned int nx=extent[xDim];
   unsigned int ny=extent[yDim];
   unsigned int nz=extent[zDim];
   unsigned int nfreq=extent[freqDim];
-  ODINLOG(odinlog,normalDebug) << "extent/n=" << (STD_string)extent << "/" << n << STD_endl;
+
+//  unsigned int n=extent.total();
+  oneframe_size_cache=nx*ny*nz*nfreq; // size of one frame
+
+  ODINLOG(odinlog,normalDebug) << "extent/oneframe_size=" << (STD_string)extent << "/" << oneframe_size_cache << STD_endl;
   resize(nx, ny, nz, nfreq); // will call reset_magnetization() and outdate_simcache()
 
   if(progmeter) progmeter->new_task(extent.total(),"Initializing Simulation");
 
 
-  xpos_cache=new float[n];
-  ypos_cache=new float[n];
-  zpos_cache=new float[n];
-  freqoffset_cache=new float[n];
-  ppm_cache=new float[n];
-  spin_density_cache=new float[n];
-  B1map_transm_cache=new STD_complex[n];
+  elapsed_time=0.0;
+  time_index_cache=0;
+  numof_time_intervals_cache=0;
+  dvector ti(sample.get_frame_durations());
+  unsigned int nti=ti.size();
+  if(nti>1) {
+    numof_time_intervals_cache=nti;
+    ODINLOG(odinlog,normalDebug) << "numof_time_intervals_cache=" << numof_time_intervals_cache << STD_endl;
+    time_intervals_cache=new double[numof_time_intervals_cache];
+    for(unsigned int iti=0; iti<numof_time_intervals_cache; iti++) time_intervals_cache[iti]=ti[iti];
+  }
+
+  farray ppmMap=sample.get_ppmMap();
+  farray spinDensity=sample.get_spinDensity();
+  farray Dcoeff=sample.get_DcoeffMap();
+  farray t1map=sample.get_T1map();
+  farray t2map=sample.get_T2map();
+
+  xpos_cache=new float[oneframe_size_cache];
+  ypos_cache=new float[oneframe_size_cache];
+  zpos_cache=new float[oneframe_size_cache];
+  freqoffset_cache=new float[oneframe_size_cache];
+
+  nframes_ppm_cache=ppmMap.size(frameDim);
+  ODINLOG(odinlog,normalDebug) << "nframes_ppm=" << nframes_ppm_cache << STD_endl;
+  ppm_cache=new float[nframes_ppm_cache*oneframe_size_cache];
+
+  nframes_spin_density_cache=spinDensity.size(frameDim);
+  ODINLOG(odinlog,normalDebug) << "nframes_spin_density=" << nframes_spin_density_cache << STD_endl;
+  spin_density_cache=new float[nframes_spin_density_cache*oneframe_size_cache];
+
+  B1map_transm_cache=new STD_complex[oneframe_size_cache];
 
   num_rec_channel_cache=1;
   if(receive_coil) num_rec_channel_cache=receive_coil->get_numof_channels();
 
 
-
   B1map_receiv_cache=new STD_complex*[num_rec_channel_cache];
   unsigned int ichan;
-  for(ichan=0; ichan<num_rec_channel_cache; ichan++) B1map_receiv_cache[ichan]=new STD_complex[n];
+  for(ichan=0; ichan<num_rec_channel_cache; ichan++) B1map_receiv_cache[ichan]=new STD_complex[oneframe_size_cache];
 
   if(sample.have_DcoeffMap) {
-    Dcoeff_cache=new float[n];
+    nframes_Dcoeff_cache=Dcoeff.size(frameDim);
+    ODINLOG(odinlog,normalDebug) << "nframes_Dcoeff=" << nframes_Dcoeff_cache << STD_endl;
+    Dcoeff_cache=new float[nframes_Dcoeff_cache*oneframe_size_cache];
     sim_diffusion=true;
   }
 
 
-  r1_cache=new float[n];
-  r2_cache=new float[n];
-  has_relax_cache=new bool[n];
+  nframes_r1_cache=t1map.size(frameDim);
+  ODINLOG(odinlog,normalDebug) << "nframes_r1=" << nframes_r1_cache << STD_endl;
+  nframes_r2_cache=t2map.size(frameDim);
+  ODINLOG(odinlog,normalDebug) << "nframes_r2=" << nframes_r2_cache << STD_endl;
+
+  r1_cache=new float[nframes_r1_cache*oneframe_size_cache];
+  r2_cache=new float[nframes_r2_cache*oneframe_size_cache];
+  has_relax_cache=new bool[oneframe_size_cache];
 
   if(magsi || sim_diffusion) {
     for(unsigned int k=0; k<4; k++) {
-      dMx[k]=new float[n];
-      dMy[k]=new float[n];
-      dMz[k]=new float[n];
-      if(k<3) dppm[k]=new float[n];
+      dMx[k]=new float[oneframe_size_cache];
+      dMy[k]=new float[oneframe_size_cache];
+      dMz[k]=new float[oneframe_size_cache];
+      if(k<3) dppm[k]=new float[oneframe_size_cache];
     }
   }
-  
+
 
   set_axes_cache(sample);
 
@@ -361,14 +420,14 @@ void SeqSimMagsi::prepare_simulation(const Sample& sample, CoilSensitivity* tran
 
   if(spat_rotmatrix) ODINLOG(odinlog,normalDebug) << "spat_rotmatrix=" << spat_rotmatrix->print() << STD_endl;
 
-  farray ppmMap=sample.get_ppmMap();
-  farray spinDensity=sample.get_spinDensity();
 
   ODINLOG(odinlog,normalDebug) << "ppmMap.get_extent()=" << ppmMap.get_extent() << STD_endl;
   ODINLOG(odinlog,normalDebug) << "spinDensity.get_extent()=" << spinDensity.get_extent() << STD_endl;
   ODINLOG(odinlog,normalDebug) << "spinDensity.sum()=" << spinDensity.sum() << STD_endl;
 
-  for(unsigned int i=0; i<n; i++) {
+  extent[frameDim]=1; // iterate over time dim for each map separately
+  unsigned int iframe;
+  for(unsigned int i=0; i<oneframe_size_cache; i++) {
     ndim indexvec=extent.index2extent(i);
 
     unsigned int ifreq=indexvec[freqDim];
@@ -404,11 +463,11 @@ void SeqSimMagsi::prepare_simulation(const Sample& sample, CoilSensitivity* tran
       zpos_cache[i]=z;
     }
 
-    ppm_cache[i]=ppmMap[i];
+    for(iframe=0; iframe<nframes_ppm_cache; iframe++) ppm_cache[iframe*oneframe_size_cache+i]=ppmMap[iframe*oneframe_size_cache+i];
 
-    spin_density_cache[i]=spinDensity[i];
+    for(iframe=0; iframe<nframes_spin_density_cache; iframe++) spin_density_cache[iframe*oneframe_size_cache+i]=spinDensity[iframe*oneframe_size_cache+i];
 
-    STD_complex b1sample=sample.get_B1map()[i];
+    STD_complex b1sample=STD_complex(1.0); // sample.get_B1map()[i];
     B1map_transm_cache[i]=b1sample;
     for(ichan=0; ichan<num_rec_channel_cache; ichan++) B1map_receiv_cache[ichan][i]=b1sample;
 
@@ -424,19 +483,30 @@ void SeqSimMagsi::prepare_simulation(const Sample& sample, CoilSensitivity* tran
       for(ichan=0; ichan<num_rec_channel_cache; ichan++) B1map_receiv_cache[ichan][i]*=receive_coil->get_sensitivity_value(ichan,xpos_cache[i],ypos_cache[i],zpos_cache[i]);
     }
 
-    if(sim_diffusion) Dcoeff_cache[i]=sample.get_DcoeffMap()[i];
+    if(sim_diffusion) {
+      for(iframe=0; iframe<nframes_Dcoeff_cache; iframe++) Dcoeff_cache[iframe*oneframe_size_cache+i]=Dcoeff[iframe*oneframe_size_cache+i];
+    }
 
-    float t1=sample.get_T1map()[i];
-    float t2=sample.get_T2map()[i];
-    if(t1<0.0) t1=0.0;
-    if(t2<0.0) t1=0.0;
-    if(t2>t1) t2=t1;
+    iframe=0;
+    while(iframe<nframes_r1_cache || iframe<nframes_r2_cache) {
 
-    has_relax_cache[i]=false;
-    if(t1>0.0) has_relax_cache[i]=true;
+      unsigned int it1 = (iframe%nframes_r1_cache)*oneframe_size_cache + i;
+      unsigned int it2 = (iframe%nframes_r2_cache)*oneframe_size_cache + i;
 
-    r1_cache[i]=secureDivision( 1.0 , t1 );
-    r2_cache[i]=secureDivision( 1.0 , t2 );
+      float t1=t1map[it1];
+      float t2=t2map[it2];
+      if(t1<0.0) t1=0.0;
+      if(t2<0.0) t1=0.0;
+      if(t2>t1) t2=t1;
+
+      if(iframe==0) has_relax_cache[i]=false; // reset only in 1st frame
+      if(t1>0.0) has_relax_cache[i]=true;
+
+      r1_cache[it1]=secureDivision( 1.0 , t1 );
+      r2_cache[it2]=secureDivision( 1.0 , t2 );
+
+      iframe++;
+    }
 
     if(magsi || sim_diffusion) {
       for(k=0; k<4; k++) {
@@ -450,14 +520,14 @@ void SeqSimMagsi::prepare_simulation(const Sample& sample, CoilSensitivity* tran
         bool calc_grads=true;
 //        if(!have_ppmMap) calc_grads=false;
 //        if(have_spinDensity && spinDensity(indexvec)<=0.0) calc_grads=false;
-        if(spinDensity(indexvec)<=0.0) calc_grads=false;
+        if(spinDensity(indexvec)<=0.0) calc_grads=false; // use only first frame
 
         if(calc_grads) {
           bool have_low=false;
           bool have_upp=false;
           float lowval=0.0;
           float uppval=0.0;
-          float currval=ppmMap(indexvec);
+          float currval=ppmMap(indexvec); // use only first frame
 
           int index=indexvec[xDim-k];
           if(index>0) {
@@ -484,11 +554,10 @@ void SeqSimMagsi::prepare_simulation(const Sample& sample, CoilSensitivity* tran
 
   update_axes();
 
-  if(!ThreadedLoop<SeqSimInterval,cvector>::init(nthreads, n)) {
+  if(!ThreadedLoop<SeqSimInterval,cvector>::init(nthreads, oneframe_size_cache)) {
     ODINLOG(odinlog,errorLog) << "cannot init multithreading" << STD_endl;
   }
 
-
   simcache_up2date=true;
 }
 
@@ -498,18 +567,35 @@ void SeqSimMagsi::prepare_simulation(const Sample& sample, CoilSensitivity* tran
 cvector SeqSimMagsi::simulate(const SeqSimInterval& simvals, double gamma) {
   Log<Seq> odinlog(this,"simulate");
 
+  cvector result(num_rec_channel_cache); // initialized with zeroes
+
   gamma_cache=gamma;
 
+  if(numof_time_intervals_cache) {
+
+    elapsed_time+=simvals.dt;
+
+    while(elapsed_time>=time_intervals_cache[time_index_cache]) {
+      elapsed_time-=time_intervals_cache[time_index_cache];
+      time_index_cache++;
+      if(time_index_cache>=numof_time_intervals_cache) time_index_cache=0;
+      ODINLOG(odinlog,normalDebug) << "Switching to time_index=" << time_index_cache << STD_endl;
+    }
+
+  }
+
   STD_vector<cvector> outvec;
   if(!execute(simvals, outvec)) {
     ODINLOG(odinlog,errorLog) << "cannot start multithreading" << STD_endl;
+    return result;
   }
 
-  cvector result(num_rec_channel_cache); // initialized with zeroes
 
   if(simvals.rec>0.0) {
     for(unsigned int i=0; i<outvec.size(); i++) {
-      result+=outvec[i];
+      if(outvec[i].size()) { // zero-duration intervals have zero size
+        result+=outvec[i];
+      }
     }
   }
   return result;
@@ -558,8 +644,8 @@ bool SeqSimMagsi::kernel(const SeqSimInterval& simvals, cvector& signal, unsigne
 
   float phase_rad=simvals.phase * PII / 180.0;
 
-  float B1amp=cabs(simvals.B1);
-  float B1phase=phase(simvals.B1);
+//  float B1amp=cabs(simvals.B1);
+//  float B1phase=phase(simvals.B1);
 
   // stuff for intra-voxel magn gradients / diffusion
   float dMxnew[4];
@@ -581,12 +667,15 @@ bool SeqSimMagsi::kernel(const SeqSimInterval& simvals, cvector& signal, unsigne
   float d_prev=0.0;
 
 
-  b1_pulse= float(gamma_cache) * B1amp * STD_complex( cos(B1phase+phase_rad), sin(B1phase+phase_rad));
+//  b1_pulse= float(gamma_cache) * B1amp * STD_complex( cos(B1phase+phase_rad), sin(B1phase+phase_rad));
+  b1_pulse= float(gamma_cache) * simvals.B1 * expc(STD_complex(0.0,phase_rad));
   ODINLOG(odinlog,normalDebug) << "b1_pulse=" << b1_pulse << STD_endl;
 
   for(unsigned int i=begin; i<end; i++) {
 
-    if(spin_density_cache[i]>0.0) {
+    float s0=spin_density_cache[(time_index_cache%nframes_spin_density_cache)*oneframe_size_cache+i];
+
+    if(s0>0.0) {
 
 
       ////////////////// Initialization ///////////////////////////
@@ -611,7 +700,7 @@ bool SeqSimMagsi::kernel(const SeqSimInterval& simvals, cvector& signal, unsigne
       b1=B1map_transm_cache[i]*b1_pulse;
       bx=b1.real();
       by=b1.imag();
-      Dw = ppm_cache[i] * w0_ppm + freqoffset_cache[i] - 2.0*PII*simvals.freq;
+      Dw = ppm_cache[(time_index_cache%nframes_ppm_cache)*oneframe_size_cache+i] * w0_ppm + freqoffset_cache[i] - 2.0*PII*simvals.freq;
       bz = gamma_cache * (simvals.Gx*xpos_cache[i] + simvals.Gy*ypos_cache[i] + simvals.Gz*zpos_cache[i]) + Dw;
 
       si_gr=co_gr=0.0;
@@ -781,8 +870,10 @@ bool SeqSimMagsi::kernel(const SeqSimInterval& simvals, cvector& signal, unsigne
         bval2+=d_prev;
         bval2*=simvals.dt;
 
-        A1= exp(-Dcoeff_cache[i] * d_prev * simvals.dt);
-        if(bval2>0.0) A2= exp(-Dcoeff_cache[i] * bval2); // Check for round-off errors when bval2 was calculated
+        float Dval=Dcoeff_cache[(time_index_cache%nframes_Dcoeff_cache)*oneframe_size_cache+i];
+
+        A1= exp(-Dval * d_prev * simvals.dt);
+        if(bval2>0.0) A2= exp(-Dval * bval2); // Check for round-off errors when bval2 was calculated
 
         Mxnew *= A2;
         Mynew *= A2;
@@ -800,8 +891,8 @@ bool SeqSimMagsi::kernel(const SeqSimInterval& simvals, cvector& signal, unsigne
       ////////////////// Relaxation ///////////////////////////////////////////////
 
       if ( has_relax_cache[i] ) { //with relaxation
-        E1=exp(-simvals.dt*r1_cache[i]);
-        E2=exp(-simvals.dt*r2_cache[i]);
+        E1=exp(-simvals.dt*r1_cache[(time_index_cache%nframes_r1_cache)*oneframe_size_cache+i]);
+        E2=exp(-simvals.dt*r2_cache[(time_index_cache%nframes_r2_cache)*oneframe_size_cache+i]);
         Mxnew *= E2;
         Mynew *= E2;
         Mznew = (Mznew-1.0) * E1 + 1.0;
@@ -860,7 +951,7 @@ bool SeqSimMagsi::kernel(const SeqSimInterval& simvals, cvector& signal, unsigne
           Mtrans *= STD_complex(weight);
         }
 
-        STD_complex sigMagn=spin_density_cache[i]*conj(Mtrans);
+        STD_complex sigMagn=s0*conj(Mtrans);
         for(ichan=0; ichan<num_rec_channel_cache; ichan++) {
           STD_complex cplxsig=sigMagn*B1map_receiv_cache[ichan][i];
           reSig[ichan]+=cplxsig.real();
@@ -957,7 +1048,7 @@ void SeqSimMonteCarlo::prepare_simulation(const Sample& sample, CoilSensitivity*
   xpixelspacing_cache=secureDivision(sample.get_FOV(xAxis),xsize);
   ypixelspacing_cache=secureDivision(sample.get_FOV(yAxis),ysize);
   zpixelspacing_cache=secureDivision(sample.get_FOV(zAxis),zsize);
-  ODINLOG(odinlog,significantDebug) << "pixelspacing=" << xpixelspacing_cache << "/" << ypixelspacing_cache << "/" << zpixelspacing_cache << STD_endl;
+  ODINLOG(odinlog,normalDebug) << "pixelspacing=" << xpixelspacing_cache << "/" << ypixelspacing_cache << "/" << zpixelspacing_cache << STD_endl;
 
   for(unsigned int i=0; i<particle.size(); i++) {
     Particle& part=particle[i];
@@ -993,6 +1084,8 @@ unsigned int SeqSimMonteCarlo::linear_index(float xpos, float ypos, float zpos)
 
 
 cvector SeqSimMonteCarlo::simulate(const SeqSimInterval& simvals, double gamma) {
+  Log<Seq> odinlog(this,"simulate");
+
   cvector result(1);
   result[0]=0.0;
 
@@ -1006,8 +1099,9 @@ cvector SeqSimMonteCarlo::simulate(const SeqSimInterval& simvals, double gamma)
   // Uniform B1 field for all particles
   float B1amp=cabs(simvals.B1);
   if(B1amp) {
-    float B1phase=phase(simvals.B1);
-    STD_complex b1_pulse= float(gamma) * B1amp * STD_complex( cos(B1phase+phase_rad), sin(B1phase+phase_rad));
+//    float B1phase=phase(simvals.B1);
+//    STD_complex b1_pulse= float(gamma) * B1amp * STD_complex( cos(B1phase+phase_rad), sin(B1phase+phase_rad));
+    STD_complex b1_pulse= float(gamma) * simvals.B1 * expc(STD_complex(0.0,phase_rad));
     float bx=b1_pulse.real();
     float by=b1_pulse.imag();
     b = norm(bx,by);
@@ -1051,6 +1145,7 @@ cvector SeqSimMonteCarlo::simulate(const SeqSimInterval& simvals, double gamma)
     Particle& part=particle[i];
 
     unsigned int j= linear_index(part.xpos, part.ypos, part.zpos);
+    ODINLOG(odinlog,normalDebug) << "linear_index[" << i << "]=" << j << STD_endl;
 
 
     ////////////////// Initialization ///////////////////////////
