@@ -9,6 +9,8 @@ unsigned int TypeTraits::typesize(const STD_string& typelabel) {
   if(typelabel==TypeTraits::type2label((s32bit)0)) return 4;
   if(typelabel==TypeTraits::type2label((float)0))  return sizeof(float);
   if(typelabel==TypeTraits::type2label((double)0)) return sizeof(double);
+  if(typelabel==TypeTraits::type2label((STD_complex)0)) return sizeof(STD_complex);
+  if(typelabel==TypeTraits::type2label((STD_string)0)) return sizeof(STD_string);
   return 0;
 }
 
